 ###############################################
#                                               #
#                 O R A C L E                   #
#       v1.0 (08/06/2009)  par MenzAgitat       #
#                                               #
#          http://www.boulets-roxx.com          #
#        IRC:  irc.epiknet.org  #boulets        #
#              irc.teepi.net    #boulets        #
#                                               #
#     Mes scripts sont tlchargeables sur      #
#            http://www.eggdrop.fr              #
#        ou  http://www.egghelp.org             #
#                                               #
 ###############################################

#
# Syntaxe : !oracle <question>
#
# Description :
# Posez votre question  l'oracle, il vous rpondra.
# Si la question a dj t pose, la rponse restera la mme.
# Oracle utilise un soundex afin de dtecter des questions identiques mme
# si l'orthographe et la ponctuation varient.
# Oracle utilise galement la mthode de la distance de Levenshtein afin
# de tolrer de petites variations et de reconnatre deux questions trs
# lgrement diffrentes comme tant une seule et mme question.
# 
# MenzAgitats_Soundex.tcl et Levenshtein.tcl sont requis par Oracle pour
# fonctionner. Assurez-vous que vous possdez bien ces deux scripts et que vous
# les avez ajouts dans le fichier eggdrop.conf AVANT Oracle.tcl :
#			source scripts/MenzAgitats_Soundex.tcl
#			source scripts/Levenshtein.tcl
#			source scripts/Oracle.tcl
#

#
#	Changelog
#		1.0 	- 1re version
#

#
# LICENCE:
#		This program is free software; you can redistribute it and/or modify
#		it under the terms of the GNU General Public License as published by
#		the Free Software Foundation; either version 2 of the License, or
#		(at your option) any later version.
#
#		This program is distributed in the hope that it will be useful,
#		but WITHOUT ANY WARRANTY; without even the implied warranty of
#		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#		GNU General Public License for more details.
#

if { [lindex [split $version] 1] < 1061800 } { putloglev o * "\00304\002\[Oracle - ERREUR\]\002\003 La version de votre eggdrop est \00304\002[lindex [split $version] 0]\002\003; Oracle.tcl ne fonctionnera correctement que sur les eggdrops version\002 1.6.18\002 ou suprieure." ; return }
if { $tcl_version < 8.4 } { putloglev o * "\00304\002\[Oracle - ERREUR\]\002\003 Oracle.tcl ncessite que \002Tcl 8.4\002 (ou plus) soit install pour fonctionner. Votre version actuelle de Tcl est \00304\002$tcl_version\002\003." ; return }
if { [lsearch -exact [package names] "MenzAgitats_Soundex"] == -1 } { putloglev o * "\00304\002\[Oracle - ERREUR\]\002\003 Oracle.tcl ncessite que \002MenzAgitat's Soundex\002 (MenzAgitats_Soundex.tcl) soit install pour fonctionner." ; return }
if { [lsearch -exact [package names] "Levenshtein"] == -1 } { putloglev o * "\00304\002\[Oracle - ERREUR\]\002\003 Oracle.tcl ncessite que \002Distance de Levenshtein\002 (Levenshtein.tcl) soit install pour fonctionner." ; return }
package require Tcl 8.4
package require MenzAgitats_Soundex
package require Levenshtein
if {[info commands Oracle::uninstall] eq "::Oracle::uninstall"} { Oracle::uninstall }
namespace eval Oracle {


	###########################
	#				PARAMETRES        #
	###########################

	# Chans sur lesquels le script sera actif (spars par un espace)
	# Attention aux majuscules, le nom du chan est sensible  la casse.
	# Si le nom du chan contient le caractre } ou { , vous devez le remplacer
	# par \} ou \{. exemple : si votre chan s'appelle #mon{chan},
	#	mettez #mon\{chan\}.
  variable allowed_chans {#boulets}

	#### COMMANDES ET AUTORISATIONS
  #
	# Commande utilise pour questionner l'oracle
	variable oraclecmd "!oracle"
	# autorisations pour la commande
	variable oracleauth "-|-"

	#### PARAMETRES DE L'ANTI-FLOOD
  #
  # Anti-flood (0 = dsactiv, 1 = activ)
  variable antiflood 1
  # Combien de commandes sont autorises en combien de temps ?
  # exemple : "6:30" = 6 commandes maximum en 30 secondes;
  # les suivantes seront ignores.
  variable cmdflood_oracle "5:60"
  # Intervalle de temps minimum entre l'affichage de 2 messages
  # avertissant que l'anti-flood a t dclench (ne rglez pas
  # cette valeur trop bas afin de ne pas tre flood par les messages
  # d'avertissement de l'anti-flood...)
  variable antiflood_msg_interval 30

	#### PARAMETRES DIVERS
	#
	# Pourcentage de chances de retourner une rponse neutre (neutral_response)
	variable neutral_rate 20
	# Tolrance maximum pour la distance de Levenshtein, en pourcentage de la
	# longueur de la phrase (ne touchez pas  cette valeur sans savoir ce que vous
	# faites)
	variable levenshtein_tolerance 20
	# Nombre maximum de questions mmorises par l'eggdrop (si une question
	# mmorise est pose plusieurs fois par la mme personne, elle donnera
	# toujours la mme rponse).
	# Afin de ne pas encombrer excessivement la mmoire, on stocke au maximum
	# $max_memory associations nick/question->rponse.
	variable max_memory 50

	#### REPONSES
	# Dans les rponses, vous pouvez utiliser certaines variables qui seront
	#	substitues par leur valeur au moment de l'affichage :
	#		$nick = nick de la personne qui a pos la question
	#		$chan = chan en cours
	#		$randnick =	nick d'une personne choisie alatoirement sur le chan
	#								(ne peut pas tre le nom de l'eggdrop ni le nick de
	#								la personne qui a pos la question)
	# Vous pouvez aussi utiliser des couleurs (selon la syntaxe habituelle
	# d'un eggdrop), du gras, du soulignement, ...
	#
	# Rponses neutres
	variable neutral_response {
		{la rponse est : 42}
		{il faudrait tre vraiment drang pour rpondre  a.}
		{c'est une question que tu devrais te poser  toi-mme.}
		{je ne sais pas.}
		{*Joker*}
		{je ne peux pas avoir rponse  tout.}
		{demande  quelqu'un d'autre.}
		{tu n'as pas besoin de le savoir.}
		{TUUUT TUUUT TUUUT TUUUT}
		{il y a des questions qu'il vaut mieux ne pas poser}
		{je ne sais pas \00313*rougit*\003}
		{...}
		{je ne rpondrai pas  a.}
		{il y a des choses qu'il vaut mieux ne pas savoir.}
		{c'est quoi ces questions ?  \037oO\037'}
		{on t'a pay pour m'emmerder ?}
		{va savoir...}
		{bah, c'est n'importe quoi.}
		{c'est incroyable le nombre de rumeurs qui peuvent circuler sur ce chan...}
		{je pourrais te rpondre mais aprs cela je serais oblig de te tuer.}
		{comment suis-je sens savoir a ? je suis une fonction alatoire dans un bot et tu le sais bien alors arrte de te faire des films et de croire  n'importe quoi.}
		{tu es trop intelligent pour demander a.}
		{donne moi de l'argent et je verrai ce que je peux faire.}
		{plus de donnes sont requises pour rpondre prcisment  cette question.}
		{merci de reformuler votre question.}
		{erreur de syntaxe}
		{Cette fonction sonne occup. Merci de ressayer dans quelques minutes...}
		{dsol, je n'coutais pas.}
		{il est totalement impossible de rpondre objectivement  cette question.}
		{je ferais mieux de ne pas te rpondre maintenant.}
		{et toi tu en penses quoi ?}
		{oublie a}
		{qui s'en soucie ?}
		{on s'en fout}
		{osef}
		{question stupide, au suivant.}
		{crois-moi, tu ne veux pas vraiment le savoir.}
		{tu n'aimerais pas connatre la rponse.}
		{la rponse ne va pas te plaire.}
		{demande donc  $randnick.}
		{$randnick ne pourra pas dire le contraire.}
		{$randnick pourrait t'en parler...}
		{je suis vraiment oblig de rpondre  toutes tes questions idiotes ?}
	}
	# Rponses oui/non
	variable yesno_response {
		{pas forcment.}
		{peut-tre dans un futur proche. Repose-moi la question demain.}
		{non.}
		{oui.}
		{peut-tre.}
		{je ne parierais pas l dessus.}
		{oui, absolument.}
		{pas ncessairement.}
		{oh que oui !}
		{oui, peut-tre.}
		{a se pourrait.}
		{les rumeurs disent que oui.}
		{videmment pas. Cesse de poser des questions stupides.}
		{non.}
		{oui.}
		{oui : 78 | non : 93 | votes blancs : 4 | abstention : 15}
		{a va pas non ?}
		{bien sr.}
		{il y a des chances.}
		{il y a peu de chances.}
		{ne dis pas n'importe quoi.}
		{sois-en sr.}
		{pas du tout.}
		{t'as qu' compter l dessus.}
		{trs certainement, oui.}
		{absolument pas.}
		{videmment, quelle question !}
		{qui t'a racont a ?!}
		{ouais et y'a ptet des pingouins sur Mars aussi...}
		{oui mais non.}
		{c'est possible en effet.}
		{la probabilit est de 1/9702831101.}
		{bien sr, a a mme t prdit par l'Institut des Risques Majeurs.}
		{ben oui, tu es bien le seul  en douter encore.}
		{et pas qu'un peu !}
		{non, pas du tout... ah... on me dit que si dans mon oreillette...}
		{si je te dis que non, tu vas pas aller te suicider ou un autre truc dingue ?}
		{ah non pas du tout.}
		{si a te fait plaisir d'y croire, alors oui.}
		{ouais}
		{seulement si tu y crois trs fort}
		{mmh je dirais que oui.}
		{dsol, rien n'est moins sr.}
		{La rponse m'apparat un peu floue mais on dirait que c'est oui.}
		{La rponse est non seulement si Saturne est aligne avec Pluton et Neptune.}
		{nop :(}
		{yep :)}
		{il semblerait que non mais c'est peut-tre un bug.}
		{si mes sources sont fiables, oui.}
		{c'est certain}
		{j'en doute}
		{tel que je le vois, ma rponse est oui.}
		{ouais, et moi je suis le Pape.}
		{c'est ridicule !}
		{tu aimerais, hein ?}
		{ouep ^^}
		{sr !}
		{oui}
		{non}
		{j'espre que tu plaisantes...}
		{mme pas en rve.}
		{seulement dans ton imagination.}
		{on dirait bien que oui.}
		{je crois que oui.}
		{grave !}
	}
	# Rponses  "comment"
	variable howshould_response {
		{en prenant beaucoup de prcautions.}
		{en prenant ton temps.}
		{tout seul.}
		{avec un ouvre-bote.}
		{en utilisant un taille-crayon}
		{aussi rapidement que possible}
		{lentement et consciencieusement.}
		{avec un ami.}
		{avec quelqu'un que tu aimes bien.}
		{ne le fais pas, c'est mieux.}
		{fais-le simplement.}
		{pas tout seul.}
		{avec de l'aide.}
		{avec un tournevis.}
		{avec une cuiller.}
		{avec une fourchette.}
		{avec un couteau.}
		{en utilisant un trombone et un chewing-gum.}
		{dans la fort.}
		{sous le lit.}
		{dans ton lit.}
		{dans la cuisine.}
		{en y pensant trs fort.}
		{avec beaucoup de courage.}
		{a mains nues.}
		{je ne sais pas comment faire a.}
		{en appelant des renforts.}
		{avec l'aide de tes parents.}
		{librement, sans te poser de questions.}
		{avec l'aide de $randnick}
	}
	# Rponses  "combien"
	variable howmany_response {
		{environ 2 ou 3.}
		{ peu prs 25.}
		{environ 20.}
		{presque 30.}
		{une bonne dizaine.}
		{pas plus de 15.}
		{pas loin de 50.}
		{une quarantaine.}
		{7 ou 8.}
		{13  mon avis.}
		{je dirais 10 au moins.}
		{42.}
		{plus de 1000 !}
	}
	# Rponses  "comment est"/"comment suis"/"est comment"/"suis comment"/...
	variable howis_response {
		{trs joli}
		{cuit  point}
		{vert  pois jaunes}
		{grand avec une moustache}
		{petit et trappu}
		{trop court}
		{trop long}
		{norme}
		{minuscule}
		{tout bleu}
		{trange}
		{mmorable}
	}
	# Rponses  "comment est"/"comment suis"/"est comment"/"sont comment"/...
	variable howare_response {
		{trs jolis}
		{cuits  point}
		{verts  pois jaunes}
		{grands avec une moustache}
		{petits et trappus}
		{trop courts}
		{trop longs}
		{normes}
		{minuscules}
		{tout bleus}
		{tranges}
		{mmorables}
	}
	# Rponses  "comment va"
	variable howgood_response {
		{trs bien}
		{pas trop bien.}
		{je ne sais pas.}
		{aucune ide.}
		{ne demande pas.}
		{demande-lui.}
		{bien}
		{mal}
	}
	# Rponses  "quand"
	variable when_response {
		{demain matin  6h.}
		{dans une heure.}
		{dans 2 mois.}
		{dans 3 mois.}
		{dans 6 mois.}
		{dans 1 an.}
		{dans 2 ans.}
		{dans 3 ans.}
		{dans 5 ans.}
		{dans 10 ans.}
		{dans 20 ans.}
		{dans 50 ans.}
		{dans 1 sicle.}
		{dans 1000 ans.}
		{dans 1 million d'annes.}
		{dans 2 heures.}
		{dans 2 jours.}
		{dans 4 jours.}
		{dans 5 jours.}
		{dans 6 jours.}
		{dans 1 semaine.}
		{dans 1 minute.}
		{dans 3 minutes.}
		{dans 5 minutes.}
		{dans 10 minutes.}
		{dans 15 minutes.}
		{dans 20 minutes.}
		{dans 30 minutes.}
		{dans moins de 2 heures.}
		{a n'arrivera pas  moins que tu n'agisses ds maintenant.}
		{dans 3 jours.}
		{aujourd'hui.}
		{maintenant.}
		{cette nuit.}
		{a s'est dj produit il y a quelques annes, tu n'avais qu' tre l.}
		{trois jours avant la 2me pleine lune aprs ton prochain anniversaire.}
		{la semaine prochaine.}
		{dans le courant de l'anne prochaine.}
		{jamais.}
		{jamais.}
		{jamais.}
		{jamais.}
		{jamais.}
		{demain aprs-midi.}
		{le mois prochain}
		{en janvier.}
		{en fvrier.}
		{en mars.}
		{en avril.}
		{en mai.}
		{en juin.}
		{en juillet.}
		{en aot.}
		{en septembre.}
		{en octobre.}
		{en novembre.}
		{en dcembre.}
		{cet t.}
		{cet hiver.}
		{au printemps.}
		{en automne.}
		{dans 5mn, si $randnick ne nous retarde pas...}
	}
	# Rponses  "pourquoi"
	variable why_response {
		{pourquoi pas ?}
		{parce que c'est comme a et puis c'est tout.}
		{pour te donner l'air intelligent.}
		{parce que c'est vachement plus marrant comme a.}
		{parce qu'on lui a demand de le faire.}
		{pour te faire parler.}
		{parce que tu le vaux bien.}
		{parce que t'es un boulet.}
		{parce que sinon a ne serait pas drle.}
		{parce que ^^}
		{parce que $randnick l'a prdit}
		{parce que $randnick a insist pour a}
	}
	# Rponses  "qui"
	variable who_response {
		{quelqu'un de bien.}
		{quelqu'un qui pose moins de questions que toi.}
		{un mutant avec des tentacules.}
		{un homme dguis en femme.}
		{le plus gros boulet connu.}
		{un illustre inconnu.}
		{c'est ta soeur.}
		{un malade mental.}
		{c'est $randnick.}
		{c'est ton pre}
		{c'est ta mre}
		{un pote de ta soeur}
		{une amie de ta mre}
		{un voisin}
		{un ami qui te veut du bien}
		{un fuyard recherch par la police}
		{un emmerdeur de premire}
	}
	# Rponses  "o"
	variable where_response {
		{dans un lit.}
		{dehors.}
		{ l'intrieur.}
		{en Italie.}
		{au Vnzuela.}
		{en Suisse.}
		{en Australie.}
		{prs de ton meilleur ami.}
		{prs de ton pire ennemi.}
		{chez toi.}
		{sur ton lieu de travail.}
		{dans le placard.}
		{sous le lit.}
		{dans la rue.}
		{dans une voiture.}
		{sur le tapis.}
		{dans un couloir.}
		{devant ton ordinateur.}
		{sur une chaise.}
		{dans le rfrigrateur.}
		{attach  un monument.}
		{dans la maison d'un ami.}
		{dans la main de quelqu'un que tu aimes bien.}
		{au fond de l'ocan.}
		{ DisneyLand.}
		{au McDo.}
		{dans les toilettes.}
		{dans une baignoire.}
		{dans l'herbe d'un pr.}
		{derrire des portes fermes.}
		{dans la 4me dimension.}
		{ ct de toi.}
		{sur la lune.}
		{quelque part dans la galaxie.}
		{quelque part o tu ne peux pas le voir.}
		{par l -->}
		{sur ce chan}
		{chez $randnick}
		{dans le frigo de $randnick}
		{DTC}
		{DTC}
		{DTC}
		{DTC}
	}
	# Rponses si la question n'en est pas une
	variable notaquestion_response {
		{si tu le dis...}
		{ok, c'est not}
		{mon boulot est de rpondre aux questions, pas d'couter tes certitudes}
		{trs intressant}
		{et la question est ... ?}
		{ok}
		{c'est bien}
		{c'est cool}
		{tant mieux}
		{chouette alors}
		{quelle bonne nouvelle !}
		{waw, c'est super intressant ce que tu nous racontes l}
		{c'est fou}
		{waw}
		{et  part a, t'avais pas une question  me poser ?}
		{rappelle-moi quelle tait la question dj ?}
	}		




####################################################################
#                                                                  #
# NE MODIFIEZ RIEN APRES CE CADRE SI VOUS NE CONNAISSEZ PAS LE TCL #
#                                                                  #
#   DO NOT MODIFY ANYTHING BELOW THIS BOX IF YOU DON'T KNOW TCL    #
#                                                                  #
####################################################################

	variable scriptname "Oracle"
  variable version "1.0.20090608"
	# inutilis, conserv au cas o
	variable cmdflood_global "5:120"
	variable floodsettingsstring [split "global $cmdflood_global oracle $cmdflood_oracle"]
	variable floodsettings ; array set floodsettings $floodsettingsstring
	variable instance ; array set instance {}
	variable antiflood_msg ; array set antiflood_msg {}
	variable soundex_table ; array set soundex_table {}
	bind pub $oracleauth $oraclecmd Oracle::ask_oracle
	bind evnt - prerehash Oracle::uninstall
	proc uninstall {args} {
		putlog "Dsallocation des ressources de \002$Oracle::scriptname...\002"
		foreach binding [lsearch -inline -all -regexp [binds *[set ns [string range [namespace current] 2 end]]*] " (::)?$ns"] {
			unbind [lindex $binding 0] [lindex $binding 1] [lindex $binding 2] [lindex $binding 4]
		}
		namespace delete ::Oracle
	}
	# Vrification de la validit des chans dfinis
	foreach element $Oracle::allowed_chans {
		if {(![validchan $element]) && ([lsearch [string tolower [channels]] [string tolower $element]] == -1)} {
			putloglev o * "\00304\002\[Oracle - WARNING\]\002\003 Le chan \002$element\002 spcifi dans la configuration n'est pas un chan enregistr sur l'eggdrop."
		}
	}
}

# procdure principale
proc Oracle::ask_oracle {nick host handle chan question} {
	if { ([lsearch -exact $Oracle::allowed_chans $chan] == -1) || (($Oracle::antiflood == 1) && ([Oracle::antiflood $chan "oracle"] == "flood")) } { return }
	if { $question eq "" } { puthelp "PRIVMSG $chan :!$nick" ; return }
	if { $question eq "?" } { puthelp "PRIVMSG $chan :!$nick ?" ; return }
	variable soundex_table
	set soundex [MAS::soundex $question]
	set clean_nick [clean_nick $nick]
	# Si l'a question pose... n'est pas une question
	if { ![string match "*\\?*" $question] } {
		set answer [lindex $Oracle::notaquestion_response [rand [llength $Oracle::notaquestion_response]]]
		set randnick [Oracle::randnick $nick $chan]
		set answer [subst $answer]
	# Si la question a dj t pose, on retrouve la rponse qui a t donne
	} elseif { [set array_entry [Oracle::compare_levenshtein $clean_nick $soundex]] != -1 } {
		set answer $soundex_table($array_entry)
	# Si la question n'a pas encore t pose, on y rpond
	} else {
		if { [rand 101] <= $Oracle::neutral_rate } {
			set answer [lindex $Oracle::neutral_response [rand [llength $Oracle::neutral_response]]]
		} elseif { ([regexp -nocase {comment (est|suis|sera)} $question]) || ([regexp -nocase {(est|suis|sera(it)?|tre) comment} $question]) } {
			set answer [lindex $Oracle::howis_response [rand [llength $Oracle::howis_response]]]
		} elseif { ([regexp -nocase {comment (sont|seront|seraient)} $question]) || ([regexp -nocase {(sont|seront|seraient) comment} $question]) } {
			set answer [lindex $Oracle::howare_response [rand [llength $Oracle::howare_response]]]
		} elseif { [string match -nocase "*comment va*" $question] } {
			set answer [lindex $Oracle::howgood_response [rand [llength $Oracle::howgood_response]]]
		} elseif { ([string match -nocase "*comment *" $question]) || ([string match -nocase "*de quelle faon*" $question]) || ([string match -nocase "*de quelle facon*" $question]) || ([string match -nocase "*de quelle manire*" $question]) || ([string match -nocase "*de quelle maniere*" $question]) || ([string match -nocase "*avec quoi*" $question]) } {
			set answer [lindex $Oracle::howshould_response [rand [llength $Oracle::howshould_response]]]
		} elseif { [string match -nocase "*pourquoi*" $question] } {
			set answer [lindex $Oracle::why_response [rand [llength $Oracle::why_response]]]
		} elseif { [string match -nocase "*quand*" $question] } {
			set answer [lindex $Oracle::when_response [rand [llength $Oracle::when_response]]]
		} elseif { ([string match -nocase "*o*" $question]) || ([string match -nocase "* quel endroit*" $question]) || ([string match -nocase "*a quel endroit*" $question]) || ([string match -nocase "*dans quel*" $question]) } {
			set answer [lindex $Oracle::where_response [rand [llength $Oracle::where_response]]]
		} elseif { [string match -nocase "*combien*" $question] } {
			set answer [lindex $Oracle::howmany_response [rand [llength $Oracle::howmany_response]]]
		} elseif { ([regexp -nocase {que(l(le)?s?)? (est|sera|sont)} $question]) || ([string match -nocase "*qu'est-ce*" $question]) || ([string match -nocase "*kess*" $question]) } {
			set answer [lindex $Oracle::neutral_response [rand [llength $Oracle::neutral_response]]]
		} elseif { ([regexp -nocase {^qui (es|a )} $question]) || ([string match -nocase "*qui \\?*" $question]) } {
			set answer [lindex $Oracle::who_response [rand [llength $Oracle::who_response]]]
		} else {
			set answer [lindex $Oracle::yesno_response [rand [llength $Oracle::yesno_response]]]
		}
		set randnick [Oracle::randnick $nick $chan]
		set answer [subst $answer]
		# si le nombre de questions stockes en mmoire atteint la limite autorise
		# ($max_memory), on supprime la plus ancienne
		if { [array size Oracle::soundex_table] >= $Oracle::max_memory } {
			set oldest [unixtime]
			foreach array_entry [array names Oracle::soundex_table] {
				if { [set time_entry [lindex [split $array_entry ""] 0]] < $oldest } { set oldest $time_entry }
			}
			unset soundex_table([lindex [set array_entries [array names Oracle::soundex_table]] [lsearch -regexp $array_entries "^[set oldest].*.*"]])
		}
		# on stocke la valeur soundex de la question, ainsi que la rponse associe
		set soundex_table([unixtime]$clean_nick$soundex) $answer
	}
	puthelp "PRIVMSG $chan :$nick > $answer"
	# on ajoute manuellement aux logs puisque le logger interne ne le fait pas
	# avec les commandes et les rponses de l'eggdrop.
	putloglev p $chan "<$nick> !oracle $question"
	putloglev p $chan "<$::botnick> $nick > $answer"
}

# Cherche des correspondances ayant une distance de Levenshtein infrieure  la
# tolrance dfinie dans la table de soundex
proc Oracle::compare_levenshtein {clean_nick soundex} {
	if { [info exists Oracle::soundex_table] } {
		foreach array_entry [array names Oracle::soundex_table] {
			if { $clean_nick eq [lindex [set split_array_entry [split $array_entry ""]] 1] } {
				set soundex_entry [lindex $split_array_entry 2]
				if { [levenshtein::distance $soundex_entry $soundex] <= [expr $Oracle::levenshtein_tolerance * [string length $soundex_entry] / 100] } {
					return $array_entry
				}
			}
		}
		return -1
	}
}

# Retourne un nick alatoire parmi les users prsents sur le chan
# (sauf l'eggdrop, sauf les autres eggs du botnet, sauf l'user qui a pos la
# question)
proc Oracle::randnick {nick chan} {
	set users_list [lreplace [set users [chanlist $chan -b|]] [set index [lsearch $users $nick]] $index]
	set users_list [lreplace $users_list [set index [lsearch $users_list $::botnick]] $index]
	return [lindex $users_list [rand [llength $users_list]]]
}

# Nettoie les caractres spciaux dans les pseudos
proc Oracle::clean_nick {nick} {
	return [string map -nocase {
		"\[" "%!" "\]" "!%"
		"\\" "!@" "\|" "@!"
		"\{" ":!" "\}" "!:"
		"\^" "!" "\$" "!"
	} $nick]
}

# Contrle du flood
proc Oracle::antiflood {chan type} {
  variable antiflood_msg
  if {![info exists antiflood_msg($chan$type)]} { set antiflood_msg($chan$type) 0 }
  variable instance
  if {![info exists instance($chan$type)]} { set instance($chan$type) 0 }
  set max_instances [lindex [split $Oracle::floodsettings($type) ":"] 0]
  set instance_length [lindex [split $Oracle::floodsettings($type) ":"] 1]
  if { $instance($chan$type) >= $max_instances } {
    if { $antiflood_msg($chan$type) == 0 } {
      set antiflood_msg($chan$type) 1
      if {$type != "global"} {
        putquick "privmsg $chan :[code 04 $chan]:::[code 14 $chan] Contrle de flood activ pour la commande [code bold $chan]!$type[code bold $chan] : pas plus de $max_instances requte(s) toutes les $instance_length secondes.[code endcolor $chan]"
      } else {
        putquick "privmsg $chan :[code 04 $chan]:::[code 14 $chan] Contrle de flood sur les commandes de [code bold $chan]Oracle[code bold $chan] : pas plus de $max_instances commandes toutes les $instance_length secondes.[code endcolor $chan]"
      }
      utimer $Oracle::antiflood_msg_interval "Oracle::antiflood_msg_reset $chan $type"
    }
    return "flood"
  } else {
    incr instance($chan$type)
    utimer $instance_length "Oracle::antiflood_close_instance $chan $type"
    return "no flood"
  }
}
proc Oracle::antiflood_close_instance {chan type} {
  variable instance
  if { ([info exists instance($chan$type)]) && ($instance($chan$type) > 0) } { incr instance($chan$type) -1 }
}
proc Oracle::antiflood_msg_reset {chan type} {
  variable antiflood_msg
  set antiflood_msg($chan$type) 0
}

# filtrage des codes de couleur/gras/soulign si le mode +c est dtect sur le chan
proc Oracle::code {code chan} {
  if {$code == "bold"} {
    if {![string match *c* [lindex [split [getchanmode $chan]] 0]]} { return "\002" } { return "" }
  } elseif {$code == "underline"} {
    if {![string match *c* [lindex [split [getchanmode $chan]] 0]]} { return "\037" } { return "" }
  } elseif {$code == "endcolor"} {
    if {![string match *c* [lindex [split [getchanmode $chan]] 0]]} { return "\003" } { return "" }
  } else {
    if {![string match *c* [lindex [split [getchanmode $chan]] 0]]} { return "\003$code" } { return "" }
  }
}


putlog "\002*$Oracle::scriptname v$Oracle::version*\002 par MenzAgitat a t charg."
